module RnAux017 where
import Test	-- Import main module so there really is a loop
		-- (avoid warning message)
data Wibble = Wibble
data Wobble = Wobble

a :: Int -> Int
a = undefined

b :: Int -> Int
b = undefined

c :: Int -> Int
c = undefined
