test('Dep1', only_ways('normal'), compile, [''])
test('Dep2', only_ways('normal'), compile, [''])
test('Dep3', only_ways('normal'), compile, [''])
test('KindEqualities', only_ways('normal'), compile, [''])
test('KindEqualities2', only_ways('normal'), compile, [''])
test('Rae31', only_ways('normal'), compile, [''])
test('RAE_T32b', only_ways('normal'), compile, [''])
test('KindLevels', normal, compile, [''])
test('RaeBlogPost', normal, compile, [''])
test('mkGADTVars', normal, compile, [''])
test('TypeLevelVec',normal,compile, [''])
test('T9632', normal, compile, [''])
# Simon says in #11330: The "simplifier ticks exhausted" error is expected;
# see Section 7 of the paper:
# http://research.microsoft.com/en-us/um/people/simonpj/papers/haskell-dynamic/
test('dynamic-paper', expect_fail_for(['optasm', 'optllvm']), compile, [''])
test('T11311', normal, compile, [''])
test('T11405', normal, compile, [''])
test('T11241', normal, compile, [''])
test('T11711', normal, compile, [''])
test('RaeJobTalk', normal, compile, [''])
test('T11635', normal, compile, [''])
test('T11719', normal, compile, [''])
