test('MonadFailWarnings', normal, compile, [''])
test('MonadFailErrors', normal, compile_fail, [''])
test('MonadFailWarningsDisabled', normal, compile, [''])
test('MonadFailWarningsWithRebindableSyntax', normal, compile, [''])
