module T6975 where
data Foo = Foo
  { field1 :: Int -> Int
  , field2 :: Int -> Int
  , field3 :: Int -> Int
  , field4 :: Int -> Int
  , field5 :: Int -> Int
  , field6 :: Int -> Int
  , field7 :: Int -> Int
  , field8 :: Int -> Int
  , field9 :: Int -> Int
  , field10 :: Int -> Int
  , field11 :: Int -> Int
  , field12 :: Int -> Int
  , field13 :: Int -> Int
  , field14 :: Int -> Int
  , field15 :: Int -> Int
  , field16 :: Int -> Int
  , field17 :: Int -> Int
  , field18 :: Int -> Int
  , field19 :: Int -> Int
  , field20 :: Int -> Int
  , field21 :: Int -> Int
  , field22 :: Int -> Int
  , field23 :: Int -> Int
  , field24 :: Int -> Int
  , field25 :: Int -> Int
  , field26 :: Int -> Int
  , field27 :: Int -> Int
  , field28 :: Int -> Int
  , field29 :: Int -> Int
  , field30 :: Int -> Int
  , field31 :: Int -> Int
  , field32 :: Int -> Int
  , field33 :: Int -> Int
  , field34 :: Int -> Int
  , field35 :: Int -> Int
  , field36 :: Int -> Int
  , field37 :: Int -> Int
  , field38 :: Int -> Int
  , field39 :: Int -> Int
  , field40 :: Int -> Int
  , field41 :: Int -> Int
  , field42 :: Int -> Int
  , field43 :: Int -> Int
  , field44 :: Int -> Int
  , field45 :: Int -> Int
  , field46 :: Int -> Int
  , field47 :: Int -> Int
  , field48 :: Int -> Int
  , field49 :: Int -> Int
  , field50 :: Int -> Int
  , field51 :: Int -> Int
  , field52 :: Int -> Int
  , field53 :: Int -> Int
  , field54 :: Int -> Int
  , field55 :: Int -> Int
  , field56 :: Int -> Int
  , field57 :: Int -> Int
  , field58 :: Int -> Int
  , field59 :: Int -> Int
  , field60 :: Int -> Int
  , field61 :: Int -> Int
  , field62 :: Int -> Int
  , field63 :: Int -> Int
  , field64 :: Int -> Int
  , field65 :: Int -> Int
  , field66 :: Int -> Int
  , field67 :: Int -> Int
  , field68 :: Int -> Int
  , field69 :: Int -> Int
  , field70 :: Int -> Int
  , field71 :: Int -> Int
  , field72 :: Int -> Int
  , field73 :: Int -> Int
  , field74 :: Int -> Int
  , field75 :: Int -> Int
  , field76 :: Int -> Int
  , field77 :: Int -> Int
  , field78 :: Int -> Int
  , field79 :: Int -> Int
  , field80 :: Int -> Int
  , field81 :: Int -> Int
  , field82 :: Int -> Int
  , field83 :: Int -> Int
  , field84 :: Int -> Int
  , field85 :: Int -> Int
  , field86 :: Int -> Int
  , field87 :: Int -> Int
  , field88 :: Int -> Int
  , field89 :: Int -> Int
  , field90 :: Int -> Int
  , field91 :: Int -> Int
  , field92 :: Int -> Int
  , field93 :: Int -> Int
  , field94 :: Int -> Int
  , field95 :: Int -> Int
  , field96 :: Int -> Int
  , field97 :: Int -> Int
  , field98 :: Int -> Int
  , field99 :: Int -> Int
  , field100 :: Int -> Int
 }
