test('AnnotatedConstraint', normal, compile_fail, [''])
test('AnnotatedConstraintNotForgotten', normal, compile_fail, [''])
test('Defaulting1MROff', normal, compile, [''])
test('ExtraConstraintsWildcardInExpressionSignature', normal, compile_fail, [''])
test('ExtraConstraintsWildcardInPatternSignature', normal, compile_fail, [''])
test('ExtraConstraintsWildcardInPatternSplice', normal, compile_fail, [''])
test('ExtraConstraintsWildcardInTypeSpliceUsed',
     [req_interp,
      extra_clean(['ExtraConstraintsWildcardInTypeSplice.o', 'ExtraConstraintsWildcardInTypeSplice.hi'])],
     multimod_compile_fail, ['ExtraConstraintsWildcardInTypeSpliceUsed', config.ghc_th_way_flags])
test('ExtraConstraintsWildcardInTypeSplice2',
     [expect_broken(11101), req_interp],
     compile_fail, [''])
test('ExtraConstraintsWildcardNotEnabled', normal, compile_fail, [''])
test('ExtraConstraintsWildcardNotLast', normal, compile_fail, [''])
test('ExtraConstraintsWildcardNotPresent', normal, compile_fail, [''])
test('ExtraConstraintsWildcardTwice', normal, compile_fail, [''])
test('Forall1Bad', normal, compile_fail, [''])
test('InstantiatedNamedWildcardsInConstraints', normal, compile_fail, [''])
test('NamedExtraConstraintsWildcard', normal, compile_fail, [''])
test('NamedWildcardExplicitForall', normal, compile_fail, [''])
test('NamedWildcardInTypeSplice', normal, compile_fail, [''])
test('NamedWildcardsEnabled', normal, compile_fail, [''])
test('NamedWildcardsNotEnabled', normal, compile_fail, [''])
test('NamedWildcardsNotInMonotype', normal, compile_fail, [''])
test('NestedExtraConstraintsWildcard', normal, compile_fail, [''])
test('NestedNamedExtraConstraintsWildcard', normal, compile_fail, [''])
test('PartialClassMethodSignature', normal, compile_fail, [''])
test('PartialClassMethodSignature2', normal, compile_fail, [''])
test('PartialTypeSignaturesDisabled', normal, compile_fail, [''])
test('ScopedNamedWildcardsBad', normal, compile_fail, [''])
test('TidyClash', normal, compile_fail, [''])
# Bug
test('TidyClash2', expect_broken(9478), compile_fail, [''])
test('UnnamedConstraintWildcard1', normal, compile_fail, [''])
test('UnnamedConstraintWildcard2', normal, compile_fail, [''])
test('WildcardInADT1', normal, compile_fail, [''])
test('WildcardInADT2', normal, compile_fail, [''])
test('WildcardInADT3', normal, compile_fail, [''])
test('WildcardInADTContext1', normal, compile_fail, [''])
test('WildcardInADTContext2', normal, compile_fail, [''])
test('WildcardInDefault', normal, compile_fail, [''])
test('WildcardInDefaultSignature', normal, compile_fail, [''])
test('WildcardInDeriving', normal, compile_fail, [''])
test('WildcardInForeignExport', normal, compile_fail, [''])
test('WildcardInForeignImport', normal, compile_fail, [''])
test('WildcardInGADT1', normal, compile_fail, [''])
test('WildcardInGADT2', normal, compile_fail, [''])
test('WildcardInInstanceHead', normal, compile_fail, [''])
test('WildcardInInstanceSig', normal, compile_fail, [''])
test('WildcardsInPatternAndExprSig', normal, compile_fail, [''])
test('WildcardInPatSynSig', normal, compile_fail, [''])
test('WildcardInNewtype', normal, compile_fail, [''])
test('WildcardInStandaloneDeriving', normal, compile_fail, [''])
test('WildcardInstantiations', normal, compile_fail, [''])
test('WildcardInTypeFamilyInstanceRHS', normal, compile_fail, [''])
test('WildcardInTypeSynonymLHS', normal, compile_fail, [''])
test('WildcardInTypeSynonymRHS', normal, compile_fail, [''])
test('T10615', normal, compile_fail, [''])
test('T10045', normal, compile_fail, [''])
test('T10999', normal, compile_fail, [''])
test('T11122', normal, compile, [''])
test('T11976', normal, compile_fail, [''])
test('T12732', normal, compile_fail, ['-fobject-code -fdefer-typed-holes'])
