module Imp100 where
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10
import {-# SOURCE #-} Int10

