
test('tcfail001', normal, compile_fail, [''])
test('tcfail002', normal, compile_fail, [''])
test('tcfail003', normal, compile_fail, [''])
test('tcfail004', normal, compile_fail, [''])
test('tcfail005', normal, compile_fail, [''])
test('tcfail006', normal, compile_fail, [''])
test('tcfail007', normal, compile_fail, [''])
test('tcfail008', normal, compile_fail, [''])
test('tcfail009', normal, compile_fail, [''])
test('tcfail010', normal, compile_fail, [''])
test('tcfail011', normal, compile_fail, [''])
test('tcfail012', normal, compile_fail, [''])
test('tcfail013', normal, compile_fail, [''])
test('tcfail014', normal, compile_fail, [''])
test('tcfail015', normal, compile_fail, [''])
test('tcfail016', normal, compile_fail, [''])
test('tcfail017', normal, compile_fail, [''])
test('tcfail018', normal, compile_fail, [''])
test('tcfail019', normal, compile_fail, [''])
test('tcfail020', normal, compile_fail, [''])
test('tcfail021', normal, compile_fail, [''])
test('tcfail023', normal, compile_fail, [''])
test('tcfail027', normal, compile_fail, [''])
test('tcfail028', normal, compile_fail, [''])
test('tcfail029', normal, compile_fail, [''])
test('tcfail030', normal, compile_fail, [''])
test('tcfail031', normal, compile_fail, [''])
test('tcfail032', normal, compile_fail, [''])
test('tcfail033', normal, compile_fail, [''])
test('tcfail034', normal, compile_fail, [''])
test('tcfail035', normal, compile_fail, [''])
test('tcfail036', normal, compile_fail, [''])
test('tcfail037', normal, compile_fail, [''])
test('tcfail038', normal, compile_fail, [''])
test('tcfail040', normal, compile_fail, [''])
test('tcfail041', normal, compile_fail, [''])
test('tcfail042', normal, compile_fail, [''])
test('tcfail043', normal, compile_fail, [''])
test('tcfail044', normal, compile_fail, [''])
test('tcfail046', normal, compile_fail, [''])
test('tcfail047', normal, compile_fail, [''])
test('tcfail048', normal, compile_fail, [''])
test('tcfail049', normal, compile_fail, [''])
test('tcfail050', normal, compile_fail, [''])
test('tcfail051', normal, compile_fail, [''])
test('tcfail052', normal, compile_fail, [''])
test('tcfail053', normal, compile_fail, [''])
test('tcfail054', normal, compile_fail, [''])
test('tcfail055', normal, compile_fail, [''])
test('tcfail056', normal, compile_fail, [''])
test('tcfail057', normal, compile_fail, [''])
test('tcfail058', normal, compile_fail, [''])
test('tcfail061', normal, compile_fail, [''])
test('tcfail062', normal, compile_fail, [''])
test('tcfail063', normal, compile_fail, [''])
test('tcfail065', normal, compile_fail, [''])
test('tcfail067', normal, compile_fail, [''])
test('tcfail068', normal, compile_fail, [''])
test('tcfail069', normal, compile_fail, [''])
test('tcfail070', normal, compile_fail, [''])
test('tcfail071', normal, compile, [''])
test('tcfail072', normal, compile_fail, [''])
test('tcfail073', normal, compile_fail, [''])
test('tcfail075', normal, compile_fail, [''])
test('tcfail076', normal, compile_fail, [''])
test('tcfail077', normal, compile_fail, [''])
test('tcfail078', normal, compile_fail, [''])
test('tcfail079', normal, compile_fail, [''])
test('tcfail080', normal, compile_fail, [''])
test('tcfail082', normal, compile_fail, [''])
test('tcfail083', normal, compile_fail, [''])
test('tcfail084', normal, compile_fail, [''])
test('tcfail085', normal, compile_fail, [''])
test('tcfail086', normal, compile_fail, [''])
test('tcfail088', normal, compile_fail, [''])
test('tcfail089', normal, compile_fail, [''])
test('tcfail090', normal, compile_fail, [''])
test('tcfail092', normal, compile_fail, [''])
test('tcfail093', normal, compile, [''])
test('tcfail094', normal, compile_fail, [''])
test('tcfail095', normal, compile_fail, [''])
test('tcfail096', normal, compile_fail, [''])
test('tcfail097', normal, compile_fail, [''])
test('tcfail098', normal, compile_fail, [''])
test('tcfail099', normal, compile_fail, [''])
test('tcfail100', normal, compile_fail, [''])
test('tcfail101', normal, compile_fail, [''])
test('tcfail102', normal, compile_fail, [''])
test('tcfail103', normal, compile_fail, [''])
test('tcfail104', normal, compile_fail, [''])
test('tcfail105', normal, compile, [''])
test('tcfail106', normal, compile_fail, [''])
test('tcfail107', normal, compile_fail, [''])
test('tcfail108', normal, compile_fail, [''])
test('tcfail109', normal, compile_fail, [''])
test('tcfail110', normal, compile_fail, [''])
test('tcfail112', normal, compile_fail, [''])
test('tcfail113', normal, compile_fail, [''])
test('tcfail114', normal, compile_fail, [''])
test('tcfail116', normal, compile_fail, [''])
test('tcfail117', normal, compile_fail, [''])
test('tcfail118', normal, compile_fail, [''])
test('tcfail119', normal, compile_fail, [''])
test('tcfail121', normal, compile_fail, [''])
test('tcfail122', normal, compile_fail, [''])
test('tcfail123', normal, compile_fail, [''])
test('tcfail124', normal, compile, [''])
test('tcfail125', normal, compile_fail, [''])
test('tcfail126', reqlib('mtl'), compile, [''])
test('tcfail127', normal, compile_fail, [''])
test('tcfail128', normal, compile_fail, [''])
test('tcfail129', normal, compile_fail, [''])
test('tcfail130', normal, compile_fail, [''])
test('tcfail131', normal, compile_fail, [''])
test('tcfail132', normal, compile_fail, [''])
test('tcfail133', normal, compile_fail, [''])
test('tcfail134', normal, compile_fail, [''])
test('tcfail135', normal, compile_fail, [''])
test('tcfail136', normal, compile_fail, [''])
test('tcfail137', normal, compile_fail, [''])

test('tcfail138', normal, compile, [''])
# Now fails; see notes in file

test('tcfail139', normal, compile_fail, [''])
test('tcfail140', normal, compile_fail, [''])
test('tcfail142', normal, compile_fail, [''])
test('tcfail143', normal, compile_fail, [''])
test('tcfail144', normal, compile, [''])
test('tcfail145', normal, compile, [''])
test('tcfail146', normal, compile_fail, [''])
test('tcfail147', normal, compile_fail, [''])
test('tcfail148', normal, compile_fail, [''])
test('tcfail149', normal, compile_and_run, [''])
test('tcfail150', normal, compile_fail, [''])
test('tcfail151', normal, compile_fail, [''])
test('tcfail152', normal, compile_fail, [''])
test('tcfail153', normal, compile_fail, [''])
test('tcfail154', normal, compile_fail, [''])
test('tcfail155', normal, compile_fail, [''])
test('tcfail156', normal, compile_fail, [''])
test('tcfail157', normal, compile_fail, [''])
test('tcfail158', normal, compile_fail, [''])
test('tcfail159', normal, compile_fail, [''])
test('tcfail160', normal, compile_fail, [''])
test('tcfail161', normal, compile_fail, [''])
test('tcfail162', normal, compile_fail, [''])
test('tcfail164', normal, compile_fail, [''])
test('tcfail165', normal, compile_fail, [''])
test('tcfail166', normal, compile_fail, [''])
test('tcfail167', normal, compile_fail, [''])
test('tcfail168', normal, compile_fail, [''])
test('tcfail169', normal, compile_fail, [''])
test('tcfail170', normal, compile_fail, [''])
test('tcfail171', normal, compile_fail, [''])
test('tcfail172', normal, compile, [''])
test('tcfail173', normal, compile_fail, [''])
test('tcfail174', normal, compile_fail, [''])
test('tcfail175', normal, compile_fail, [''])
test('tcfail176', normal, compile_fail, [''])
test('tcfail177', normal, compile_fail, [''])
test('tcfail178', normal, compile_fail, [''])
test('tcfail179', normal, compile_fail, [''])
test('tcfail180', normal, compile_fail, [''])
test('tcfail181', normal, compile_fail, [''])
test('tcfail182', normal, compile_fail, [''])
test('tcfail183', normal, compile_fail, [''])
test('tcfail184', normal, compile_fail, [''])
test('tcfail185', normal, compile_fail, [''])
test('tcfail186',
     extra_clean(['Tcfail186_Help.hi', 'Tcfail186_Help.o']),
     multimod_compile_fail, ['tcfail186', '-v0'])
test('tcfail187', normal, compile_fail, [''])

test('tcfail188', normal, compile, [''])

test('tcfail189', normal, compile_fail, [''])
test('tcfail190', normal, compile_fail, [''])
test('tcfail191', normal, compile_fail, [''])
test('tcfail193', normal, compile_fail, [''])
test('tcfail195', normal, compile_fail, [''])
test('tcfail196', normal, compile_fail, [''])
test('tcfail197', normal, compile_fail, [''])
test('tcfail198', normal, compile_fail, [''])
test('tcfail199', normal, compile_fail, [''])
test('tcfail200', normal, compile_fail, [''])
test('tcfail201', normal, compile_fail, [''])
test('tcfail202', normal, compile_fail, [''])
test('tcfail203', normal, compile_fail, [''])
test('tcfail203a', normal, compile_fail, [''])
test('tcfail204', normal, compile_fail, [''])
test('tcfail206', normal, compile_fail, [''])

test('T1595', normal, compile_fail, [''])
test('T1899', normal, compile_fail, [''])
test('T2126', normal, compile_fail, [''])
test('T2307', normal, compile_fail, [''])
test('T2414', normal, compile_fail, [''])
test('T2538', normal, compile_fail, [''])
test('T2688', normal, compile_fail, [''])
test('T2714', normal, compile_fail, [''])
test('T2994', normal, compile_fail, [''])
test('T3155', normal, compile_fail, [''])
test('T3176', normal, compile_fail, [''])
test('T1633', normal, compile_fail, [''])
test('T2806', normal, compile_fail, [''])
test('T3323', normal, compile_fail, [''])
test('T3406', normal, compile_fail, [''])
test('T3540', normal, compile_fail, [''])
test('T3102', normal, compile_fail, [''])
test('T3613', normal, compile_fail, [''])
test('fd-loop', normal, compile_fail, [''])
test('T3950', normal, compile_fail, [''])
test('T3966', normal, compile_fail, ['-O'])
test('IPFail', normal, compile_fail, [''])

test('T3468', extra_clean(['T3468.hi-boot', 'T3468.o-boot', 'T3468a.hi', 'T3468a.o']),
     multimod_compile_fail, ['T3468', '-v0'])
test('T2846b', normal, compile_fail, [''])
test('FrozenErrorTests', normal, compile_fail, [''])
test('SCLoop', normal, compile_fail, [''])
test('T4875', normal, compile_fail, [''])
test('mc19', normal, compile_fail, [''])
test('mc20', normal, compile_fail, [''])
test('mc21', normal, compile_fail, [''])
test('mc22', normal, compile_fail, [''])
test('mc23', normal, compile_fail, [''])
test('mc24', normal, compile_fail, [''])
test('mc25', normal, compile_fail, [''])
test('tcfail207', normal, compile_fail, [''])
test('T5084', normal, compile_fail, [''])
test('tcfail208', normal, compile_fail, [''])
test('tcfail209', normal, compile_fail, [''])
test('tcfail209a', normal, compile_fail, [''])
test('tcfail210', normal, compile_fail, [''])
test('tcfail211', normal, compile_fail, [''])
test('tcfail212', normal, compile_fail, [''])
test('tcfail213', normal, compile_fail, [''])
test('tcfail214', normal, compile_fail, [''])
test('tcfail215', normal, compile_fail, [''])
test('tcfail216', normal, compile_fail, [''])
test('tcfail217', normal, compile_fail, [''])
test('tcfail218', normal, compile_fail, [''])
test('tcfail219', normal, multimod_compile_fail, ['tcfail219.hsig', '-sig-of "ShouldFail is base:Data.Bool"'])
test('tcfail220', normal, multimod_compile_fail, ['tcfail220.hsig', '-sig-of "ShouldFail is base:Prelude"'])
test('tcfail221', normal, multimod_compile_fail, ['tcfail221.hsig', '-sig-of "ShouldFail is base:Prelude"'])
test('tcfail222', normal, multimod_compile_fail, ['tcfail222.hsig', '-sig-of "ShouldFail is base:Data.STRef"'])
test('tcfail223', normal, compile_fail, [''])
test('tcfail224', normal, compile_fail, [''])

test('SilentParametersOverlapping', normal, compile, [''])
test('FailDueToGivenOverlapping', normal, compile_fail, [''])
test('LongWayOverlapping', normal, compile_fail, [''])
test('T5051', normal, compile, [''])
test('T5236',normal,compile,[''])
test('T5246',normal,compile_fail,[''])
test('T5300',normal,compile_fail,[''])
test('T5095',normal,compile_fail,[''])
test('T1897a',normal,compile_fail,[''])

test('AssocTyDef01', normal, compile_fail, [''])
test('AssocTyDef02', normal, compile_fail, [''])
test('AssocTyDef03', normal, compile_fail, [''])
test('AssocTyDef04', normal, compile_fail, [''])
test('AssocTyDef05', normal, compile_fail, [''])
test('AssocTyDef06', normal, compile_fail, [''])
test('AssocTyDef07', normal, compile_fail, [''])
test('AssocTyDef08', normal, compile_fail, [''])
test('AssocTyDef09', normal, compile_fail, [''])
test('T3592', normal, compile_fail, [''])
test('T5570', normal, compile_fail, [''])
test('T5691', normal, compile_fail, [''])
test('T5689', normal, compile_fail, [''])
test('T5684', normal, compile_fail, [''])
test('T5858', normal, compile_fail, [''])
test('T5957', normal, compile_fail, [''])
test('T6001', normal, compile_fail, [''])
test('T6022', normal, compile_fail, [''])
test('T5853', normal, compile_fail, [''])
test('T6018fail', extra_clean([ 'T6018fail.hi' , 'T6018fail.o'
                              , 'T6018Afail.hi', 'T6018Afail.o'
                              , 'T6018Bfail.hi', 'T6018Bfail.o'
                              , 'T6018Cfail.hi', 'T6018Cfail.o'
                              , 'T6018Dfail.hi', 'T6018Dfail.o'])
    , multimod_compile_fail, ['T6018fail', '-no-hs-main -c'])
test('T6018failclosed', normal, compile_fail, [''])
test('T6018failclosed2', normal, compile_fail, [''])
test('T6078', normal, compile_fail, [''])
test('FDsFromGivens', normal, compile, [''])
test('FDsFromGivens2', normal, compile_fail, [''])
test('T7019', normal, compile_fail,[''])
test('T7019a', normal, compile_fail,[''])
test('T5978', normal, compile_fail, [''])
test('TcMultiWayIfFail', when(compiler_lt('ghc', '7.5'), skip), compile_fail, [''])
test('T2534', normal, compile_fail, [''])
test('T7175', normal, compile_fail, [''])
test('T7210', normal, compile_fail, [''])
test('T6161', normal, compile_fail, [''])
test('T7368', normal, compile_fail, [''])
test('T7264', normal, compile_fail, [''])
test('T6069', normal, compile_fail, [''])
test('T7410', normal, compile_fail, [''])
test('T7453', normal, compile_fail, [''])
test('T7525', normal, compile_fail, [''])
test('T7368a', normal, compile_fail, [''])
test('T7545', normal, compile, [''])
test('T7279', normal, compile_fail, [''])
test('T2247', normal, compile_fail, [''])
test('T7609', normal, compile_fail, [''])
test('T7645', normal, compile_fail, [''])
test('T2354', normal, compile_fail, ['-O'])
test('T7734', normal, compile_fail, [''])
test('T7697', normal, compile_fail, [''])
test('T7696', normal, compile_fail, [''])
test('T7748a', normal, compile_fail, [''])
test('TcNoNullaryTC', when(compiler_lt('ghc', '7.7'), skip), compile_fail, [''])
test('TcNullaryTCFail', when(compiler_lt('ghc', '7.7'), skip), compile_fail, [''])
test('T7857', normal, compile_fail, [''])
test('T7778', normal, compile_fail, [''])
test('T7851', normal, compile_fail, [''])
test('T7856', normal, compile_fail, [''])
test('T7869', normal, compile_fail, [''])
test('T7892', normal, compile_fail, [''])
test('T7809', normal, compile_fail, [''])
test('T7989', normal, compile_fail, [''])
test('T8034', normal, compile_fail, [''])
test('T8142', normal, compile_fail, [''])
test('T8262', normal, compile_fail, [''])

# TcCoercibleFail times out with the compiler is compiled with -DDEBUG.
# This is expected (see comment in source file).
test('TcCoercibleFail', [when(compiler_debugged(), skip)], compile_fail, [''])

test('TcCoercibleFail2', [], compile_fail, [''])
test('TcCoercibleFail3', [], compile_fail, [''])
test('T8306', normal, compile_fail, [''])
test('T8392a', normal, compile, [''])
test('T8428', normal, compile_fail, [''])
test('T8450', normal, compile_fail, [''])
test('T8514', normal, compile_fail, [''])
test('ContextStack1', normal, compile_fail, ['-freduction-depth=10'])
test('ContextStack2', normal, compile_fail, ['-freduction-depth=10'])
test('T8570', extra_clean(['T85570a.o', 'T8570a.hi','T85570b.o', 'T8570b.hi']),
     multimod_compile_fail, ['T8570', '-v0'])
test('T8603', normal, compile_fail, [''])
test('TcStaticPointersFail01',
     when(compiler_lt('ghc', '7.9'), skip), compile_fail, [''])
test('TcStaticPointersFail02',
     when(compiler_lt('ghc', '7.9'), skip), compile_fail, [''])
test('TcStaticPointersFail03',
     when(compiler_lt('ghc', '7.9'), skip), compile_fail, [''])
test('T8806', normal, compile_fail, [''])
test('T8912', normal, compile_fail, [''])
test('T9033', normal, compile_fail, [''])
test('T8883', normal, compile_fail, [''])
test('T9196', normal, compile_fail, [''])
test('T9260', normal, compile_fail, [''])
test('T9305', normal, compile_fail, [''])
test('T9323', normal, compile_fail, [''])
test('T9415', normal, compile_fail, [''])
test('T9612', normal, compile_fail, [''])
test('T9634', normal, compile_fail, [''])
test('T9739', normal, compile_fail, [''])
test('T9774', normal, compile_fail, [''])
test('T9318', normal, compile_fail, [''])
test('T9201', normal, compile_fail, [''])
test('T9109', normal, compile_fail, [''])
test('T9497d', normal, compile_fail, ['-fdefer-type-errors -fno-defer-typed-holes'])
test('T8044', normal, compile_fail, [''])
test('T4921', normal, compile_fail, [''])
test('T9605', normal, compile_fail, [''])
test('T9999', normal, compile_fail, [''])
test('T10194', normal, compile_fail, [''])
test('T8030', normal, compile_fail, [''])
test('T9858a', normal, compile_fail, [''])
test('T9858b', normal, compile_fail, [''])
test('T9858e', normal, compile_fail, [''])

test('T10285',
     extra_clean(['T10285a.hi', 'T10285a.o']),
     multimod_compile_fail, ['T10285', '-v0'])
test('T10351', normal, compile_fail, [''])
test('T10534', extra_clean(['T10534a.hi', 'T10534a.o']),
     multimod_compile_fail, ['T10534', '-v0'])
test('T10495', normal, compile_fail, [''])
test('VtaFail', normal, compile_fail, [''])

test('ExpandSynsFail1', normal, compile_fail, ['-fprint-expanded-synonyms'])
test('ExpandSynsFail2', normal, compile_fail, ['-fprint-expanded-synonyms'])
test('ExpandSynsFail3', normal, compile_fail, ['-fprint-expanded-synonyms'])
test('ExpandSynsFail4', normal, compile_fail, ['-fprint-expanded-synonyms'])
test('T10698', expect_broken(10698), compile_fail, [''])
test('T10836', normal, compile_fail, [''])
test('T10715', normal, compile, [''])
test('T10715b', normal, compile_fail, [''])
test('T10971b', normal, compile_fail, [''])
test('T10971d', extra_clean(['T10971c.hi', 'T10971c.o']), multimod_compile_fail, ['T10971d','-v0'])
test('CustomTypeErrors01', normal, compile_fail, [''])
test('CustomTypeErrors02', normal, compile_fail, [''])
test('CustomTypeErrors03', normal, compile_fail, [''])
test('CustomTypeErrors04', normal, compile_fail, [''])
test('CustomTypeErrors05', normal, compile_fail, [''])
test('T11112', normal, compile_fail, [''])
test('ClassOperator', normal, compile_fail, [''])
test('T11274', normal, compile_fail, [''])
test('T10619', normal, compile_fail, [''])
test('T11347', normal, compile_fail, [''])
test('T11356', normal, compile_fail, [''])
test('T11355', normal, compile_fail, [''])
test('T11464', normal, compile_fail, [''])
test('T11563', normal, compile_fail, [''])
test('T11313', normal, compile_fail, [''])
test('T11623', normal, compile_fail, [''])
test('T11723', normal, compile_fail, [''])
test('T11724', normal, compile_fail, [''])
test('BadUnboxedTuple', normal, compile_fail, [''])
test('T12151', normal, compile_fail, [''])
test('T7437', normal, compile_fail, [''])
test('T11947a', normal, compile_fail, [''])
test('T11974b', normal, compile_fail, [''])
test('T12406', normal, compile_fail, [''])
test('T12177', normal, compile_fail, [''])
test('T12170a', normal, compile_fail, [''])
test('T11990a', normal, compile_fail, [''])
test('T11990b', normal, compile_fail, [''])
test('T12124', normal, compile_fail, [''])
test('T12529', normal, compile_fail, [''])
test('T12589', normal, compile_fail, [''])
