setTestOpts([extra_run_opts('-ignore-dot-ghci'),
             normalise_slashes])

test('print001', normal, ghci_script, ['print001.script'])
test('print002', normal, ghci_script, ['print002.script'])
test('print003', normal, ghci_script, ['print003.script'])
test('print004', normal, ghci_script, ['print004.script'])
test('print005', normal, ghci_script, ['print005.script'])
test('print006', normal, ghci_script, ['print006.script'])
test('print007', normal, ghci_script, ['print007.script'])
test('print008', normal, ghci_script, ['print008.script'])
test('print009', normal, ghci_script, ['print009.script'])
test('print010', normal, ghci_script, ['print010.script'])
test('print011', normal, ghci_script, ['print011.script'])
test('print012', normal, ghci_script, ['print012.script'])
test('print013', normal, ghci_script, ['print013.script'])
test('print014', normal, ghci_script, ['print014.script'])
test('print016', normal, ghci_script, ['print016.script'])
test('print017', normal, ghci_script, ['print017.script'])
test('print018', normal, ghci_script, ['print018.script'])
test('print019', normal, ghci_script, ['print019.script'])
test('print020', normal, ghci_script, ['print020.script'])
test('print021', normal, ghci_script, ['print021.script'])
test('print022', normal, ghci_script, ['print022.script'])
test('print023', normal, ghci_script, ['print023.script'])
test('print024', normal, ghci_script, ['print024.script'])
test('print025', normal, ghci_script, ['print025.script'])
test('print026', normal, ghci_script, ['print026.script'])
test('print027', normal, ghci_script, ['print027.script'])
test('print028', normal, ghci_script, ['print028.script'])
test('print029', normal, ghci_script, ['print029.script'])
test('print030', normal, ghci_script, ['print030.script'])
test('print031', normal, ghci_script, ['print031.script'])
test('print032', normal, ghci_script, ['print032.script'])
test('print033', normal, ghci_script, ['print033.script'])
test('print034', normal, ghci_script, ['print034.script'])
test('print035',
     [extra_clean(['../Unboxed.hi', '../Unboxed.o'])],
     ghci_script, ['print035.script'])
test('print036', when(compiler_ge('ghc','7.8'), expect_broken(9046)), ghci_script, ['print036.script'])

test('break001', normal, ghci_script, ['break001.script'])
test('break002', normal, ghci_script, ['break002.script'])
test('break003', normal, ghci_script, ['break003.script'])
test('break005', normal, ghci_script, ['break005.script'])
test('break006', normal, ghci_script, ['break006.script'])
test('break007', extra_clean(['Break007.o', 'Break007.hi']),
     ghci_script, ['break007.script'])
test('break008', normal, ghci_script, ['break008.script'])
test('break009', combined_output, ghci_script, ['break009.script'])
test('break010', normal, ghci_script, ['break010.script'])
test('break011', combined_output, ghci_script, ['break011.script'])
test('break012', normal, ghci_script, ['break012.script'])
test('break013', normal, ghci_script, ['break013.script'])
test('break014', normal, ghci_script, ['break014.script'])
test('break015', expect_broken(1532), ghci_script, ['break015.script'])
test('break016', combined_output, ghci_script, ['break016.script'])
test('break017', combined_output, ghci_script, ['break017.script'])
test('break018', normal, ghci_script, ['break018.script'])
test('break019', normal, ghci_script, ['break019.script'])
test('break020', normal, ghci_script, ['break020.script'])
test('break021', normal, ghci_script, ['break021.script'])
test('break024', combined_output, ghci_script, ['break024.script'])
test('break025', normal, ghci_script, ['break025.script'])
test('break026', normal, ghci_script, ['break026.script'])
test('break027', normal, ghci_script, ['break027.script'])
test('break028', normal, ghci_script, ['break028.script'])

test('dynbrk001', normal, ghci_script, ['dynbrk001.script'])
test('dynbrk002', normal, ghci_script, ['dynbrk002.script'])
test('dynbrk003', normal, ghci_script, ['dynbrk003.script'])
test('dynbrk004', normal, ghci_script, ['dynbrk004.script'])
test('dynbrk005', expect_broken(1530), ghci_script, ['dynbrk005.script'])
test('dynbrk007', normal, ghci_script, ['dynbrk007.script'])
test('dynbrk008', normal, ghci_script, ['dynbrk008.script'])
test('dynbrk009', normal, ghci_script, ['dynbrk009.script'])

test('result001', normal, ghci_script, ['result001.script'])

test('listCommand001', combined_output, ghci_script, ['listCommand001.script'])
test('listCommand002', normal, ghci_script, ['listCommand002.script'])

test('hist001', normal, ghci_script, ['hist001.script'])

test('T2740', normal, ghci_script, ['T2740.script'])

test('getargs', normal, ghci_script, ['getargs.script'])
test('T7386', normal, ghci_script, ['T7386.script'])
test('T8557', normal, ghci_script, ['T8557.script'])
test('T12458', normal, ghci_script, ['T12458.script'])
