test('bidir', normal, compile, [''])
test('overlap', normal, compile, [''])
test('univ', normal, compile, [''])
test('ex', normal, compile, [''])
test('ex-prov', normal, compile, [''])
test('ex-view', normal, compile, [''])
test('ex-num', normal, compile, [''])
test('num', normal, compile, [''])
test('incomplete', normal, compile, [''])
test('export', normal, compile, [''])
test('T8966', normal, compile, [''])
test('T9023', normal, compile, [''])
test('unboxed-bind-bang', normal, compile, [''])
test('T9732', normal, compile, [''])
test('T8584-1', normal, compile, [''])
test('T8584-2', normal, compile, [''])
test('T8584-3', normal, compile, [''])
test('T8968-1', normal, compile, [''])
test('T8968-2', normal, compile, [''])
test('T8968-3', normal, compile, [''])
test('ImpExp_Imp', [extra_clean(['ImpExp_Exp.hi', 'ImpExp_Exp.o'])], multimod_compile, ['ImpExp_Imp', '-v0'])
test('T9857', normal, compile, [''])
test('T9889', normal, compile, [''])
test('T9867', normal, compile, [''])
test('T9975a', normal, compile_fail, [''])
test('T9975b', normal, compile, [''])
test('T10747', normal, compile, [''])
test('T10997', [extra_clean(['T10997a.hi', 'T10997a.o'])], multimod_compile, ['T10997', '-v0'])
test('T10997_1', [extra_clean(['T10997_1a.hi', 'T10997_1a.o'])], multimod_compile, ['T10997_1', '-v0'])
test('records-compile', normal, compile, [''])
test('records-poly', normal, compile, [''])
test('records-req', normal, compile, [''])
test('records-prov-req', normal, compile, [''])
test('records-req-only', normal, compile, [''])
test('pat-syn-bundle', [extra_clean(['BundleInternal1.hi', 'BundleInternal1.o'])], multimod_compile, ['Bundle1', '-v0'])
test('pat-syn-trans-bundle', [extra_clean(['Bundle.hi', 'Bundle.o', 'BundleInternal.hi', 'BundleInternal.o'])], multimod_compile, ['TransBundle', '-v0'])
test('ExportSyntaxImport', [extra_clean(['ExportSyntax.hi', 'ExportSyntax.o'])], multimod_compile, ['ExportSyntaxImport', '-v0'])
test('poly-export', normal, compile, [''])
test('poly-export2', normal, compile, [''])
test('poly-export3', normal, compile, [''])
test('multi-export', normal, compile, [''])
test('export-super-class', normal, compile, [''])
test('export-record-selector', normal, compile, [''])
test('T10897', normal, multi_compile, ['T10897', [
                                       ('T10897a.hs','-c')
                                      ], '-v0'])
test('T11213', normal, compile, [''])
test('T11224b', normal, compile, [''])
test('MoreEx', normal, compile, [''])
test('T11283', normal, compile, [''])
test('T11367', normal, compile, [''])
test('T11351', normal, compile, [''])
test('T11633', normal, compile, [''])
test('T12094', normal, compile, [''])
test('T12484', normal, compile, [''])
test('T12489', normal, compile, [''])
test('T11959', expect_broken(11959), multimod_compile, ['T11959', '-v0'])
test('T11987', normal, multimod_compile, ['T11987', '-v0'])
test('T12615', normal, compile, [''])
test('T12698', normal, compile, [''])
test('T12746', normal, multi_compile, ['T12746', [('T12746A.hs', '-c')],'-v0'])
