test('WarnMinimal', normal, compile, [''])
test('WarnMinimalFail1', normal, compile_fail, [''])
test('WarnMinimalFail2', normal, compile_fail, [''])
test('WarnMinimalFail3', normal, compile_fail, [''])
