test('annotations', normal, run_command, ['$MAKE -s --no-print-directory annotations'])
test('parseTree',   normal, run_command, ['$MAKE -s --no-print-directory parseTree'])
test('comments',    normal, run_command, ['$MAKE -s --no-print-directory comments'])
test('exampleTest', normal, run_command, ['$MAKE -s --no-print-directory exampleTest'])
test('listcomps',   normal, run_command, ['$MAKE -s --no-print-directory listcomps'])
test('T10255',      normal, run_command, ['$MAKE -s --no-print-directory T10255'])
test('T10268',      normal, run_command, ['$MAKE -s --no-print-directory T10268'])
test('T10269',      normal, run_command, ['$MAKE -s --no-print-directory T10269'])
test('T10280',      normal, run_command, ['$MAKE -s --no-print-directory T10280'])
test('T10312',      normal, run_command, ['$MAKE -s --no-print-directory T10312'])
test('T10307',      normal, run_command, ['$MAKE -s --no-print-directory T10307'])
test('T10309',      normal, run_command, ['$MAKE -s --no-print-directory T10309'])
test('boolFormula', normal, run_command, ['$MAKE -s --no-print-directory boolFormula'])
test('T10357',      normal, run_command, ['$MAKE -s --no-print-directory T10357'])
test('T10358',      normal, run_command, ['$MAKE -s --no-print-directory T10358'])
test('T10278',      normal, run_command, ['$MAKE -s --no-print-directory T10278'])
test('T10354',      normal, run_command, ['$MAKE -s --no-print-directory T10354'])
test('T10396',      normal, run_command, ['$MAKE -s --no-print-directory T10396'])
test('T10399',      normal, run_command, ['$MAKE -s --no-print-directory T10399'])
test('T10313',      normal, run_command, ['$MAKE -s --no-print-directory T10313'])
test('T11018',      normal, run_command, ['$MAKE -s --no-print-directory T11018'])
test('bundle-export', normal, run_command, ['$MAKE -s --no-print-directory bundle-export'])
test('T10276',      normal, run_command, ['$MAKE -s --no-print-directory T10276'])
test('T11321',      normal, run_command, ['$MAKE -s --no-print-directory T11321'])
test('T11332',      normal, run_command, ['$MAKE -s --no-print-directory T11332'])
test('T11430',      normal, run_command, ['$MAKE -s --no-print-directory T11430'])
