#test('haddockE001', normal, compile_fail, ['-haddock'])
#test('haddockE002', normal, compile_fail, ['-haddock'])
#test('haddockE003', normal, compile_fail, ['-haddock'])
test('haddockE004', normal, compile_fail, ['-haddock'])
