test('WCompatWarningsOn',    normal, compile, [''])
test('WCompatWarningsOff',   normal, compile, [''])
test('WCompatWarningsNotOn', normal, compile, [''])
test('WCompatWarningsOnOff', normal, compile, [''])
